using System;
using System.Collections;
using System.Windows.Forms;

namespace Client
{
	/// <summary>
	/// A GUI manager for the election server. This object instantiates the
	/// window to allow the user to enter the name of the election.
	/// Once the list of the candidates is received for this election,
	/// this objects calls on VoteDisplay to show available candidates and
	/// allow the user to pick one. When it has to do some work, the 
	/// ClientDisplay calls a Client methods. As part of its processing, 
	/// the Client then calls some mixture of the appropriate routines to 
	/// display the results of the operation. 
	/// </summary>
	public class ClientDisplay 
	{

		private Client client;
		private ArrayList list;
		private ElNameDisplay elName;
		private VoteDisplay voteDisp;

		/// <summary>
		/// Constructor. Instantiates a new window to allow the user enter
		/// the name of the election
		/// </summary>
		/// <param name="client">Client Manager </param>
		public ClientDisplay(Client client)
		{
			this.client = client;
			list = null;
			elName = new ElNameDisplay(client, this);
			Application.Run(elName);
		}
		
		/// <summary>
		/// Method called to notify the user about errors
		/// </summary>
		/// <param name="text">Error Message</param>
		public void error(string text)
		{
			MessageBox.Show(text, "Error");
		}

		/// <summary>
		/// Friendly Notification
		/// </summary>
		/// <param name="text"></param>
		public void notify(string text)
		{
			MessageBox.Show(text, "Info");
		}

		/// <summary>
		/// Adds Candidates to be displayed later
		/// </summary>
		/// <param name="name">Name of the candidate</param>
		public void putCandidate(string name)
		{
			this.list.Add(name);
		}

		/// <summary>
		/// Method called by the ElNameDisplay to display the candidates
		/// </summary>
		/// <param name="election">Name of the election</param>
		public void chooseCandidate(string election)
		{
					
			list = new ArrayList();
			try 
			{
				client.displayCandidates(election, this);
			}
					
			catch (DisplayException e) 
			{
				error(e.Message);//e
				Application.Exit();
			}
			catch (Exception e) 
			{
				error(e.Message);
				Application.Exit();
			}

			voteDisp = new VoteDisplay(list, election, client);
			voteDisp.Show();
			
		}
	}
}
