using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;

namespace Server
{
	/// <summary>
	/// A GUI for the election server manager. This is what the election 
	/// official interacts with. When it has to do some work, the GUI calls 
	/// a Server method. As part of its processing, the server then calls 
	/// some mixture of the appropriate routines to display the 
	/// results of the operation. The particular display implementation 
	/// does the work it needs to maintain state between calls by the server 
	/// itself. 
	/// </summary>
	public class ServerDisplay : System.Windows.Forms.Form
	{
		private System.Windows.Forms.ListBox listBox1;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Button button_Open;
		private System.Windows.Forms.Button button_Close;
		private System.Windows.Forms.Button button_View;
		private System.Windows.Forms.Button button_Exit;
			
		private Server server;
		private ArrayList vector;
		private ArrayList votes; //VoteTable
		private String title;
		private String winner;
		private int nVotes;
	
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="server">Server parameter, used to provide 
		/// the functionality </param>
		public ServerDisplay(Server server)
		{
			this.server=server;
			vector = null;
			votes = null;
			title = null;
			winner = null;
			
		}
		
		/// <summary>
		/// Initiliazes the GUI
		/// </summary>
		public void start()
		{
			InitializeComponent();
			vector = new ArrayList();
			server.displayElections(this);
			string[] st = new string[vector.Count];
			for (int i=0; i<vector.Count; i++)
				st[i] = (string)vector[i];
			this.listBox1.Items.AddRange(st);
			
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

	#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ServerDisplay));
			this.listBox1 = new System.Windows.Forms.ListBox();
			this.label1 = new System.Windows.Forms.Label();
			this.button_Open = new System.Windows.Forms.Button();
			this.button_Close = new System.Windows.Forms.Button();
			this.button_View = new System.Windows.Forms.Button();
			this.button_Exit = new System.Windows.Forms.Button();
			this.SuspendLayout();
			
			this.listBox1.Location = new System.Drawing.Point(8, 24);
			this.listBox1.Name = "listBox1";
			this.listBox1.Size = new System.Drawing.Size(312, 56);
			this.listBox1.TabIndex = 0;
			
			this.label1.Location = new System.Drawing.Point(104, 0);
			this.label1.Name = "label1";
			this.label1.TabIndex = 1;
			this.label1.Text = "Elections:";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// button_Open
			// 
			this.button_Open.Location = new System.Drawing.Point(8, 96);
			this.button_Open.Name = "button_Open";
			this.button_Open.TabIndex = 2;
			this.button_Open.Text = "Open";
			this.button_Open.Click += new System.EventHandler(this.button_Open_Click);
			// 
			// button_Close
			// 
			this.button_Close.Location = new System.Drawing.Point(88, 96);
			this.button_Close.Name = "button_Close";
			this.button_Close.TabIndex = 3;
			this.button_Close.Text = "Close";
			this.button_Close.Click += new System.EventHandler(this.button_Close_Click);
			// 
			// button_View
			// 
			this.button_View.Location = new System.Drawing.Point(168, 96);
			this.button_View.Name = "button_View";
			this.button_View.TabIndex = 4;
			this.button_View.Text = "View";
			this.button_View.Click += new System.EventHandler(this.button_View_Click);
			// 
			// button_Exit
			// 
			this.button_Exit.Location = new System.Drawing.Point(248, 96);
			this.button_Exit.Name = "button_Exit";
			this.button_Exit.TabIndex = 5;
			this.button_Exit.Text = "Exit";
			this.button_Exit.Click += new System.EventHandler(this.button_Exit_Click);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(326, 124);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.button_Exit,
																		  this.button_View,
																		  this.button_Close,
																		  this.button_Open,
																		  this.label1,
																		  this.listBox1});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D;
			//this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.Name = "Form1";
			this.Text = "Election Server Manager";
			this.ResumeLayout(false);
		}
	#endregion


		/// <summary>
		/// Open Election Button listener
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="ev"></param>
		private void button_Open_Click(object sender, System.EventArgs ev)
		{
			string election = (String) this.listBox1.SelectedItem;
			if (election == null)
				error("To open an election, you you must select one from the list.");
			else 
			{
				try 
				{
					//Elections with '*' in the front are open
					int index = (int)this.listBox1.SelectedIndex;
					if (election[0] == '*')
						election = election.Substring(1,election.Length-1);
					server.openElection(election);
					notify("Election '"+election+"' has been opened.");
					this.SuspendLayout();
					this.listBox1.Items[index] = "*"+election;
					this.ResumeLayout(false);

				}
				catch (DisplayException e) 
				{
					error(e.Message);
				}
				catch (UndefinedElectionException e) 
				{
					error("An internal error has occured:" + e.Message);
					Application.Exit();
				}
			}

		}

		/// <summary>
		/// Close Election Button listener
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="ev"></param>
		private void button_Close_Click(object sender, System.EventArgs ev)
		{
			string election = (String) this.listBox1.SelectedItem;
			if (election == null)
			{
				error("To close an election, you you must select one from the list.");
				return;
			}
			else 
			{
				try 
				{
					//Elections with '*' in the front are open
					int index = (int)this.listBox1.SelectedIndex;
					votes = new ArrayList();
					if (election[0] == '*')
                        election = election.Substring(1,election.Length-1);
					server.closeElection(election, this);
					this.SuspendLayout();
					this.listBox1.Items[index] = election;
					this.ResumeLayout(false);
				}
				catch (DisplayException e) 
				{
					error(e.Message);
					return;
				}
				catch (UndefinedElectionException e) 
				{
					error("An internal error has occured."  + e.Message);
					Application.Exit();
				}
			}
			resultsWindow();

		}

		/// <summary>
		/// View Election Results Button listener
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="ev"></param>
		private void button_View_Click(object sender, System.EventArgs ev)
		{
			string election = (String) this.listBox1.SelectedItem;
			if (election == null)
			{
				error("To view an election, you you must select one from the list.");
				return;
			}
			else 
			{
				try 
				{
					//Elections with '*' in the front are open
					if (election[0] == '*')
						election = election.Substring(1,election.Length-1);
					votes = new ArrayList();
					server.viewElection(election, this);
				}
				catch (DisplayException e) 
				{
					error(e.Message);
					return;
				}
				catch (UndefinedElectionException e) 
				{
					error(e.Message);
					Application.Exit();
				}
			}
			resultsWindow();
		}

		/// <summary>
		/// Exit Application Button listener
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void button_Exit_Click(object sender, System.EventArgs e)
		{
			this.Dispose();
			Application.Exit();
			return;
		}

		/// <summary>
		/// Instantiates a new window with election results
		/// </summary>
		private void resultsWindow()
		{
			DisplayResults disp = new DisplayResults(votes, title, winner, nVotes);
			disp.Show();
		}

		/// <summary>
		/// Puts the title for the election to be displayed
		/// </summary>
		/// <param name="title">Election title</param>
		public void putTitle(string title)
		{
			this.title = title;
		}

		/// <summary>
		/// Puts the winner for the election to be displayed
		/// </summary>
		/// <param name="name">Election title</param>
		/// <param name="votes">Number of votes casted for the winner</param>
		public void putWinner(string name, int votes)
		{
			this.winner = name;
			this.nVotes = votes;
		}

		/// <summary>
		/// Puts the candidate in the vector to be displayed
		/// </summary>
		/// <param name="name">Election title</param>
		/// <param name="votes">Number of votes casted for the candidate</param>
		public void putCandidate(string name, int votes)
		{
			this.votes.Add(new VoteTable(name, votes));
		}

		/// <summary>
		/// Puts the election to be displayed
		/// </summary>
		/// <param name="name">Name of the election</param>
		public void putElection(string name)
		{
			vector.Add(name);
		}

		/// <summary>
		/// Displays error messages
		/// </summary>
		/// <param name="text">Error Message</param>
		public void error(string text)
		{
			MessageBox.Show(text, "Error");
		}

		/// <summary>
		/// Displays a message to the user
		/// </summary>
		/// <param name="text">Message</param>
		public void notify(String text)
		{
			MessageBox.Show(text, "Info");
		}

		/// <summary>
		/// Opens the election
		/// </summary>
		/// <param name="election">Election Name</param>
		public void openElection(string election)
		{
			if (election == null)
				error("To open an election, you you must select one from the list.");
			else 
			{
				try 
				{
					server.openElection(election);
					notify("Election '"+election+"' has been opened.");
				}
				catch (DisplayException e) 
				{
					error(e.Message);
				}
				catch (UndefinedElectionException e) 
				{
					error("An internal error has occured." + e.Message);
					Application.Exit();
				}
			}

		}
	}	
}
	
	

	


