using System;
using System.Collections;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;


namespace Shared
{
	/// <summary>
	/// Message object using to allow easy communication between the 
	/// client and the server by using Serialization. Message consists of
	/// a type defined below and a ArrayList of the items.
	/// </summary>
	
	[Serializable]
	public class Message
	{
		/// <summary>
		/// Miscellaneous error, not currently used for anything
		/// </summary>
		public  const int ERROR = 0;             
		
		/// <summary>
		/// Bad election name given (closed or nonexistent)
		/// </summary>
		public const int BAD_ELECTION = 1;

		/// <summary>
		/// Bad candidate name given for vote request
		/// </summary>
		public const int BAD_CANDIDATE = 2;

		/// <summary>
		/// Give me the list of candidates for the named election
		/// </summary>
		public const int QUERY_CANDIDATES = 3;

		/// <summary>
		/// Here is the list of candidates
		/// </summary>
		public const int LIST_CANDIDATES = 4;

		/// <summary>
		/// Here is my vote for election + candidate
		/// </summary>
		public const int SEND_VOTE = 5;

		/// <summary>
		/// We got your vote 
		/// </summary>
		public const int ACK_VOTE = 6;
  
		// Type of this message
		public int type;

		/** Fields of the message */
		public ArrayList items;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="type">Type of the message</param>
		public Message(int type) 
		{
			this.type = type;
			this.items = null;
		}
  
		/// <summary>
		/// Constructor with the fields given
		/// </summary>
		/// <param name="type">Type of Message</param>
		/// <param name="items">List of items being sent</param>
		public Message(int type, ArrayList items)
		{
			this.type = type;
			this.items = items;
		}
  	}
}
